@@ -25,7 +25,7 @@ use POSIX qw( SIGTERM WIFEXITED WEXITSTATUS WIFSIGNALED WTERMSIG );
 use Socket qw( sockaddr_family AF_UNIX );
 use Time::HiRes qw( time );
 
-our $VERSION = '0.47';
+our $VERSION = '0.48';
 
 # Abstract Units of Time
 use constant AUT => $ENV{TEST_QUICK_TIMERS} ? 0.1 : 1;
@@ -373,10 +373,76 @@ Tests the Loop's ability to handle timer events
 
 =cut
 
-use constant count_tests_timer => 12;
+use constant count_tests_timer => 21;
 sub run_tests_timer
 {
    my $done = 0;
+   # New watch/unwatch API
+
+   $loop->watch_time( after => 2 * AUT, code => sub { $done = 1; } );
+
+   is_oneref( $loop, '$loop has refcount 1 after watch_time' );
+
+   time_between {
+      my $now = time;
+      $loop->loop_once( 5 * AUT );
+
+      # poll might have returned just a little early, such that the TimerQueue
+      # doesn't think anything is ready yet. We need to handle that case.
+      while( !$done ) {
+         die "It should have been ready by now" if( time - $now > 5 * AUT );
+         $loop->loop_once( 0.1 * AUT );
+      }
+   } 1.5, 2.5, 'loop_once(5) while waiting for watch_time after';
+
+   $loop->watch_time( at => time + 2 * AUT, code => sub { $done = 2; } );
+
+   time_between {
+      my $now = time;
+      $loop->loop_once( 5 * AUT );
+
+      # poll might have returned just a little early, such that the TimerQueue
+      # doesn't think anything is ready yet. We need to handle that case.
+      while( !$done ) {
+         die "It should have been ready by now" if( time - $now > 5 * AUT );
+         $loop->loop_once( 0.1 * AUT );
+      }
+   } 1.5, 2.5, 'loop_once(5) while waiting for watch_time at';
+
+   my $cancelled_fired = 0;
+   my $id = $loop->watch_time( after => 1 * AUT, code => sub { $cancelled_fired = 1 } );
+   $loop->unwatch_time( $id );
+   undef $id;
+
+   $loop->loop_once( 2 * AUT );
+
+   ok( !$cancelled_fired, 'unwatched watch_time does not fire' );
+
+   $loop->watch_time( after => -1, code => sub { $done = 1 } );
+
+   $done = 0;
+
+   time_between {
+      $loop->loop_once while !$done;
+   } 0, 0.1, 'loop_once while waiting for negative interval timer';
+
+   my $doneA;
+   my $doneB;
+
+   $id = $loop->watch_time( after => 1 * AUT, code => sub {
+      $loop->unwatch_time( $id ); undef $id;
+      $doneA++;
+   });
+
+   $loop->watch_time( after => 1.1 * AUT, code => sub { $doneB++ } );
+
+   $loop->loop_once( 1 * AUT ) for 1 .. 3;
+
+   is( $doneA, 1, 'Self-cancelling timer still fires' );
+   is( $doneB, 1, 'Other timers still fire after self-cancelling one' );
+
+   # Legacy enqueue/requeue/cancel API
+   $done = 0;
 
    $loop->enqueue_timer( delay => 2 * AUT, code => sub { $done = 1; } );
 
@@ -394,8 +460,25 @@ sub run_tests_timer
       }
    } 1.5, 2.5, 'loop_once(5) while waiting for timer';
 
-   my $cancelled_fired = 0;
-   my $id = $loop->enqueue_timer( delay => 1 * AUT, code => sub { $cancelled_fired = 1 } );
+   # Check that short delays are achievable in one ->loop_once call
+   foreach my $delay ( 0.001, 0.01, 0.1 ) {
+      my $done;
+      my $count = 0;
+      my $start = time;
+
+      $loop->enqueue_timer( delay => $delay, code => sub { $done++ } );
+
+      while( !$done ) {
+         $loop->loop_once( 1 );
+         $count++;
+         last if time - $start > 5; # bailout
+      }
+
+      is( $count, 1, "One ->loop_once(1) sufficient for a single $delay second timer" );
+   }
+
+   $cancelled_fired = 0;
+   $id = $loop->enqueue_timer( delay => 1 * AUT, code => sub { $cancelled_fired = 1 } );
    $loop->cancel_timer( $id );
    undef $id;
 
@@ -425,29 +508,6 @@ sub run_tests_timer
    } 1.5, 2.5, 'requeued timer of delay 2';
 
    is( $done, 2, '$done is 2 after requeued timer' );
-
-   $loop->enqueue_timer( delay => -1, code => sub { $done = 1 } );
-
-   $done = 0;
-
-   time_between {
-      $loop->loop_once while !$done;
-   } 0, 0.1, 'loop_once while waiting for negative interval timer';
-
-   my $doneA;
-   my $doneB;
-
-   $id = $loop->enqueue_timer( delay => 1 * AUT, code => sub {
-      $loop->cancel_timer( $id ); undef $id;
-      $doneA++;
-   });
-
-   $loop->enqueue_timer( delay => 1.1 * AUT, code => sub { $doneB++ } );
-
-   $loop->loop_once( 1 * AUT ) for 1 .. 3;
-
-   is( $doneA, 1, 'Self-cancelling timer still fires' );
-   is( $doneB, 1, 'Other timers still fire after self-cancelling one' );
 }
 
 =head2 signal
@@ -529,7 +589,7 @@ Tests the Loop's support for idle handlers
 
 =cut
 
-use constant count_tests_idle => 10;
+use constant count_tests_idle => 11;
 sub run_tests_idle
 {
    my $called = 0;
@@ -566,11 +626,20 @@ sub run_tests_idle
 
    is( $called, 2, 'unwatched deferral not called' );
 
-   $loop->later( sub { $called = 3 } );
+   $id = $loop->watch_idle( when => 'later', code => sub { $called = 3 } );
+   my $timer_id = $loop->watch_time( delay => 5, code => sub {} );
 
    $loop->loop_once( 1 );
 
-   is( $called, 3, '$loop->later shortcut works' );
+   is( $called, 3, '$loop->later still invoked with enqueued timer' );
+
+   $loop->unwatch_time( $timer_id );
+
+   $loop->later( sub { $called = 4 } );
+
+   $loop->loop_once( 1 );
+
+   is( $called, 4, '$loop->later shortcut works' );
 }
 
 =head2 child
@@ -656,7 +725,7 @@ sub run_tests_control
 
    time_between { $loop->loop_once( 2 * AUT ) } 1.5, 2.5, 'loop_once(2) when idle';
 
-   $loop->enqueue_timer( delay => 0.1, code => sub { $loop->stop( result => "here" ) } );
+   $loop->watch_time( after => 0.1, code => sub { $loop->stop( result => "here" ) } );
 
    local $SIG{ALRM} = sub { die "Test timed out before ->stop" };
    alarm( 1 );
@@ -667,14 +736,14 @@ sub run_tests_control
 
    is_deeply( \@result, [ result => "here" ], '->stop arguments returned by ->run' );
 
-   $loop->enqueue_timer( delay => 0.1, code => sub { $loop->stop( result => "here" ) } );
+   $loop->watch_time( after => 0.1, code => sub { $loop->stop( result => "here" ) } );
 
    my $result = $loop->run;
 
    is( $result, "result", 'First ->stop argument returned by ->run in scalar context' );
 
-   $loop->enqueue_timer( delay => 0.1, code => sub {
-      $loop->enqueue_timer( delay => 0.1, code => sub { $loop->stop( "inner" ) } );
+   $loop->watch_time( after => 0.1, code => sub {
+      $loop->watch_time( after => 0.1, code => sub { $loop->stop( "inner" ) } );
       my @result = $loop->run;
       $loop->stop( @result, "outer" );
    } );
@@ -683,7 +752,7 @@ sub run_tests_control
 
    is_deeply( \@result, [ "inner", "outer" ], '->run can be nested properly' );
 
-   $loop->enqueue_timer( delay => 0.1, code => sub { $loop->loop_stop } );
+   $loop->watch_time( after => 0.1, code => sub { $loop->loop_stop } );
 
    local $SIG{ALRM} = sub { die "Test timed out before ->loop_stop" };
    alarm( 1 );
